﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Diagnostics;

namespace StwXmlTestingService
{
    public static class StwTrace
    {
        public const string TRACE_SWITCH_NAME = "StwGlobalTraceSwitch";

        private static TraceSwitch traceSwitch = new TraceSwitch(TRACE_SWITCH_NAME, String.Empty, "Info");

        public static bool TraceError
        {
            get { return traceSwitch.TraceError; }
        }

        public static bool TraceWarning
        {
            get { return traceSwitch.TraceWarning; }
        }

        public static bool TraceInfo
        {
            get { return traceSwitch.TraceInfo; }
        }

        public static bool TraceVerbose
        {
            get { return traceSwitch.TraceVerbose; }
        }

        [Conditional("TRACE")]
        public static void Write(string message)
        {
            string traceRecord = String.Format("{0}\t{1}", DateTime.Now.ToString("dd-MM-yyyy HH:mm:ss"), message);
            Trace.WriteLine(traceRecord);
            Trace.Flush();
        }

        [Conditional("TRACE")]
        public static void Write(string message, TraceLevel level)
        {
            if (traceSwitch.Level >= level)
            {
                message = String.Format("{0}\t{1}", level.ToString().ToUpper(), message);
                Write(message);
            }
        }

        [Conditional("TRACE")]
        public static void WriteError(string message)
        {
            Write(message, TraceLevel.Error);
        }

        public static void WriteWarning(string message)
        {
            Write(message, TraceLevel.Warning);
        }

        [Conditional("TRACE")]
        public static void WriteInfo(string message)
        {
            Write(message, TraceLevel.Info);
        }

        [Conditional("TRACE")]
        public static void WriteVerbose(string message)
        {
            Write(message, TraceLevel.Verbose);
        }
    }
}