﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Web;
using System.Text;
using System.IO;

namespace StwXmlTestingService
{
    /// <summary>
    /// Summary description for _default
    /// </summary>
    public class _default : IHttpHandler
    {
        private const string PARAM_FILE = "file";
        private const string PARAM_ACTION = "action";
        private const string PARAM_ACTION_DOWNLOAD = "download";
        private const string PARAM_ACTION_UPLOAD = "upload";

        private const string FOLDER_UPLOAD = "upload";
        private const string FOLDER_DOWNLOAD = "download";

        private const string KEY_STATE = "state";

        public void ProcessRequest(HttpContext context)
        {
            string paramFile = context.Request.QueryString[PARAM_FILE];
            string paramAction = context.Request.QueryString[PARAM_ACTION];
            bool error = false;

            if (String.IsNullOrEmpty(paramFile))
            {
                TraceHelp(PARAM_FILE);
                error = true;
            }

            if (String.IsNullOrEmpty(paramAction))
            {
                TraceHelp(PARAM_ACTION);
                error = true;
            }

            if (error)
            {
                SendResponse(context, "error");
                return;
            }

            if (paramAction == PARAM_ACTION_UPLOAD)
            {
                SaveXmlRequest(context);
                SendResponse(context, this.RequestedResponseState);
            }
            else if (paramAction == PARAM_ACTION_DOWNLOAD)
            {
                SendXml(context);
            }
        }

        public bool IsReusable
        {
            get
            {
                return false;
            }
        }

        public string RequestedResponseState
        {
            get { return ConfigurationManager.AppSettings[KEY_STATE]; }
        }

        private void TraceHelp(string badParamName)
        {
            StwTrace.WriteError(String.Format("URL parameter '{0}' is not supplied. Correct URL format: http://<server_path>/default.ashx?{1}=<upload|download>&{2}=<file name>",
                badParamName, PARAM_ACTION, PARAM_FILE));
        }

        private void SendResponse(HttpContext context, string status)
        {
            context.Response.ContentType = "text/xml";

            string response = "<?xml version=\"1.0\" encoding=\"Windows-1250\"?>" 
                            + "<rsp:responsePack xmlns:rsp=\"http://www.stormware.cz/schema/response.xsd\" version=\"1.0\" state=\"{0}\" application=\"StwXmlTestingService\">" 
                            +  "</rsp:responsePack>";

            response = String.Format(response, status);

            context.Response.Write(response);
        }

        private void SaveXmlRequest(HttpContext context)
        {
            StwTrace.WriteInfo("Saving XML - start");

            try
            {
                int contentLength = context.Request.ContentLength;
                byte[] content = context.Request.BinaryRead(contentLength);
                string xmlString = Encoding.UTF8.GetString(content);

                string fileName = GetXmlRequestFileName(context, GetUploadFolderName(context));
                StwTrace.WriteInfo(String.Format("Writing to XML file: {0}", fileName));

                using (FileStream fs = new FileStream(fileName, FileMode.Create, FileAccess.Write))
                {
                    fs.Write(content, 0, contentLength);
                }
            }
            catch (Exception ex)
            {
                StwTrace.WriteError(ex.Message);
            }

            StwTrace.WriteInfo("Saving XML - finished");
        }

        private void SendXml(HttpContext context)
        {
            StwTrace.WriteInfo("Reading XML - start");

            try
            {
                string fileName = GetXmlRequestFileName(context, GetDownloadFolderName(context));
                StwTrace.WriteInfo(String.Format("Reading XML file: {0}", fileName));
                context.Response.ContentType = "text/xml";

                using (FileStream fs = new FileStream(fileName, FileMode.Open, FileAccess.Read))
                {
                    long len = fs.Length;
                    const int BUFFER_SIZE = 4096;
                    int readed = 0;
                    do
                    {
                        byte[] buffer = new byte[BUFFER_SIZE];
                        readed = fs.Read(buffer, 0, BUFFER_SIZE);
                        context.Response.OutputStream.Write(buffer, 0, readed);
                    } while (readed == BUFFER_SIZE);
                }
            }
            catch (Exception ex)
            {
                StwTrace.WriteError(ex.Message);
            }

            StwTrace.WriteInfo("Reading XML - finished");
        }

        private string GetUploadFolderName(HttpContext context)
        {
            return Path.Combine(context.Request.PhysicalApplicationPath, FOLDER_UPLOAD);
        }

        private string GetDownloadFolderName(HttpContext context)
        {
            return Path.Combine(context.Request.PhysicalApplicationPath, FOLDER_DOWNLOAD);
        }

        private string GetXmlRequestFileName(HttpContext context, string folder)
        {
            string paramFile = context.Request.QueryString[PARAM_FILE];

            if (String.IsNullOrEmpty(paramFile))
                throw new ApplicationException(String.Format("Param {0} not found", PARAM_FILE));

            if (Path.GetExtension(paramFile) == String.Empty)
                paramFile += ".xml";

            string path = Path.Combine(folder, paramFile);
            return path;
        }
    }
}